/** @file
 * @brief PadicoTM core module public interface
 * @todo should be documented...
 * @ingroup TaskManager
 */

#ifndef PADICO_TM_H
#define PADICO_TM_H

#include <Padico/Puk.h>
#include "PM2.h"
#ifdef PUKABI
#include <Padico/Puk-ABI.h>
#endif

#ifdef __cplusplus
extern "C" {
#endif

#ifndef PUKABI
#include <errno.h>

  static inline void puk_spinlock_acquire(void)
  {  }
  static inline void puk_spinlock_release(void)
  {  }
  static inline void puk_abi_disable_preload(void)
  { }

#define PUK_ABI_FSYS_WRAP(SYM) SYM
#define PUK_ABI_FSYS_REAL(SYM) SYM
#define PUK_ABI_WRAP(SYM)      SYM
#define PUK_ABI_REAL(SYM)      SYM

#define puk_abi_set_virtual(F, PTR) { }

#define __puk_abi_wrap__errno errno
  static inline void puk_abi_seterrno(int e)
  {
    errno = e;
  }
  static inline void puk_abi_real_seterrno(int e)
  {
    errno = e;
  }
#define puk_abi_real_errno() errno

#endif /* PUKABI */

  /* ******************************************************* */
  /* ** main entry points */

  void padico_tm_init(void);

  void padico_tm_main(void);

  void padico_tm_finalize(void);

  void padico_tm_ready_wait(void);

  void padico_tm_ready_signal(void);

#include "tm-Tasks.h"

  /* ******************************************************* */

/** @addtogroup PadicoTasks
 * @{
 */

  /* *** padico_req_* */

  typedef void (*padico_task_notifier_t)(padico_rc_t, void*);
  typedef uint32_t padico_req_id_t;
#define PADICO_REQ_ID_NONE ((padico_req_id_t)0)

  PUK_LIST_TYPE(padico_req,
		padico_req_id_t id;
		int         completed;
		padico_rc_t rc;
		padico_task_notifier_t notify;
		void*                  notify_key;
		);
  PUK_VECT_TYPE(padico_reqs, padico_req_t);

  padico_rc_t  padico_tm_req_wait(padico_req_t req);
  padico_req_t padico_tm_req_new(padico_task_notifier_t notifier, void*key);
  void         padico_tm_req_notify(padico_req_id_t req_id, padico_rc_t rc);


  /* ** RTTASK -- a revoir... */

  enum padico_rttask_kind_e
    {
      PADICO_RTTASK_BGTHREAD,
      PADICO_RTTASK_RTTHREAD
    };

  typedef void*(*padico_thread_fun_t)(void*);

  PUK_LIST_TYPE(padico_rttask,
		enum padico_rttask_kind_e kind;
		padico_thread_fun_t background_fun;
		void*background_arg;
		);
  
  void padico_tm_rttask_submit(padico_rttask_t rttask);

  /* ** TASK */
  
  enum padico_mtask_kind_e
  {
    PADICO_TM_MOD_LOAD   = 1,
    PADICO_TM_MOD_UNLOAD = 2,
    PADICO_TM_MOD_RUN    = 3,
    PADICO_TM_MOD_START  = 4
  };

  struct padico_task_s
  {
    padico_req_t task_req;
    puk_mod_t parent;            /**< parent mod that issued the task */
    enum
      {
	PADICO_TM_TASK_EXIT = 1, /**< exit PadicoTM */
	PADICO_TM_TASK_MOD  = 2, /**< operation on modules */
	PADICO_TM_TASK_CMD  = 3, /**< execute an XML command */
	PADICO_TM_TASK_FILE = 4, /**< load a XML file and execute its content */
	PADICO_TM_TASK_MAIN = 5  /**< execute the given function in the main thread */
      } kind;
    union
    {
      /* task MOD */
      struct
      {
	enum padico_mtask_kind_e kind;
	char* mod_name;
	int   argc;
	char**argv;
      } mtask;
      /* task CMD */
      struct
      {
	char*cmd;
	int  trust_level;
      } cmdtask;
      /* task FILE */
      struct
      {
	char*file;
      } ftask;
      /* task MAIN */
      struct
      {
	int(*func)(int argc, char**argv);
	int argc;
	char**argv;
      } mainthread;
    } tinfo;
  };
  typedef struct padico_task_s*padico_task_t;

  padico_rc_t padico_tm_mod_action_args(enum padico_mtask_kind_e kind,
					const char*mod_name,
					int argc, char**argv);

  static inline padico_rc_t padico_tm_mod_action(enum padico_mtask_kind_e kind, const char*mod_name)
  {
    assert(kind != PADICO_TM_MOD_RUN && kind != PADICO_TM_MOD_START);
    padico_rc_t rc = padico_tm_mod_action_args(kind, mod_name, 0, NULL);
    return rc;
  }

  void padico_tm_task_enqueue_from(padico_task_t task, puk_mod_t parent);
#define padico_tm_task_enqueue(TASK) padico_tm_task_enqueue_from((TASK), padico_module_self())

  void padico_tm_exit(void);

#define padico_tm_invoke_later(FUNC, KEY) { padico_tm_invoke_later_named((FUNC), (KEY), #FUNC); }
  void padico_tm_invoke_later_named(padico_thread_fun_t f, void*key, const char*name);

  /** @} */

  /* thread-friendly sleeping */

  int  padico_tm_nanosleep(const struct timespec*req, struct timespec*rem);
  void padico_tm_usleep(long usec);
  void padico_tm_msleep(long msec);
  unsigned int padico_tm_sleep(unsigned long seconds);
  void padico_tm_yield(void);

  /* helper function to change return code */

  void      padico_tm_update_rc(int rc);

#ifdef __cplusplus
} /* extern "C" */
#endif

#endif /* PADICO_TM_H */
