/** @file
 * bootstrap PadicoTM as a library
 */

#include <stdio.h>

#include <Padico/Puk.h>
#include <Padico/PadicoTM.h>
#include <Padico/Module.h>

static int  padico_bootlib_init(void);
static void padico_bootlib_finalize(void);

PADICO_MODULE_DECLARE(PadicoBootLib);
PADICO_MODULE_INIT(padico_bootlib_init);
PADICO_MODULE_FINALIZE(padico_bootlib_finalize);

/* ********************************************************* */

static marcel_t main_thread;

static void*bootlib_main_thread(void*dummy)
{
  padico_out(5, "# PadicoBootLib: starting main...\n");
  padico_tm_main();
  padico_out(5, "# PadicoBootLib: main started.\n");
  return NULL;
}

static int padico_bootlib_init(void)
{
  padico_out(5, "# PadicoBootLib: init...\n");

  /* defeat LD_PRELOAD inheritance! */
  puk_abi_disable_preload();

  if(!padico_puk_initialized())
    {
      padico_fatal("PadicoBootLib: Puk not loaded.");
    }

  padico_tm_init();
  padico_out(5, "# PadicoBootLib: init done.\n");

  marcel_create(&main_thread, NULL, &bootlib_main_thread, NULL);

  padico_out(5, "# PadicoBootLib: waiting ready signal...\n");
  padico_tm_ready_wait();
  padico_out(5, "# PadicoBootLib: ready!\n");

  return 0;
}


static void padico_bootlib_finalize(void)
{
  padico_out(5, "# PadicoBootLib: exit.\n");
  padico_tm_finalize();
}

