/** @file psm2-discover.c
 *  @brief Intel PSM2 detection
 */

#include <Padico/Puk.h>
#include <Padico/Module.h>
#include <Padico/Topology.h>
#include <psm2.h>

PADICO_MODULE_DECLARE(PSM2_discover);
PADICO_MODULE_INIT(padico_psm2_init);
PADICO_MODULE_FINALIZE(padico_psm2_finalize);

/* ********************************************************* */

static uint64_t dummy_addr = 0; /* dummy address; real PSM2 addressing system uses ibverbs */

static void padico_psm2_discover(void)
{
  int ver_major = PSM2_VERNO_MAJOR;
  int ver_minor = PSM2_VERNO_MINOR;
  int rc = psm2_init(&ver_major, &ver_minor);
  if(rc != PSM2_OK)
    {
      padico_warning("PSM2: cannot init PSM2 library.\n");
      return;
    }
  uint32_t num_units = -1;
  rc = psm2_ep_num_devunits(&num_units);
  if(rc == PSM2_OK)
    {
      padico_print("PSM2: detected %d devices.\n", num_units);
      const char*s_uuid = padico_getenv("PADICO_BOOT_UUID");
      padico_string_t s_net = padico_string_new();
      padico_string_printf(s_net, "PSM2-[%s]", s_uuid);
      padico_topo_network_t network = padico_na_network_getbyid(padico_string_get(s_net));
      if(network == NULL)
	{
	  network = padico_topo_network_create(padico_string_get(s_net), PADICO_NA_SCOPE_HOST, sizeof(uint64_t), AF_UNSPEC);
	}
      padico_string_delete(s_net);
      padico_na_host_bind(padico_na_getlocalhost(), network, &dummy_addr);
    }
  else
    {
      padico_print("PSM2: no device found.\n");
    }
}

static int padico_psm2_init(void)
{
  padico_psm2_discover();
  return 0;
}

static void padico_psm2_finalize(void)
{
  /* don't call psm2_finalize() */
}
