/** @file
 * @brief Puk automatic lists definition
 * @ingroup Puk
 */

#include <assert.h>

#ifndef PUK_LIST_H

/* ** Puk lists ******************************************** */

/**@hideinitializer
 * Generates a new type for list of elements */
#define PUK_LIST_TYPE(ENAME, DECL)		\
  PUK_LIST_DECLARE_TYPE(ENAME)			\
  PUK_LIST_CREATE_TYPE(ENAME, DECL)		\
  PUK_LIST_CREATE_FUNCS(ENAME)

#define PUK_LIST_ALIGN __attribute__((aligned((sizeof(void*)))))

/**@hideinitializer
 * Forward type declaration for lists with explicit cell type */
#define PUK_LIST_DECLARE_TYPE2(ENAME, CELL)				\
  typedef struct ENAME##_list_s* ENAME##_list_t;			\
  typedef CELL*ENAME##_cell_t;						\
  typedef CELL ENAME##_cell_content_t;					\
  /** @internal type for the list itself */				\
  struct ENAME##_list_s							\
  {									\
    ENAME##_cell_t _head PUK_LIST_ALIGN; /**< @internal head of the list */ \
    ENAME##_cell_t _tail PUK_LIST_ALIGN; /**< @internal tail of the list */ \
  };									\
  /** automatic Puk type: ENAME##_itor_t is an iterator on lists of ENAME##_cell_t */ \
  typedef ENAME##_cell_t ENAME##_itor_t;

/** declare a type list with implicit cell type */
#define PUK_LIST_DECLARE_TYPE(ENAME)			\
  PUK_LIST_DECLARE_TYPE2(ENAME, struct ENAME##_s)

/** list link to use with use-supplied cell type */
#define PUK_LIST_LINK(ENAME)						\
  struct ENAME##_list_link_s						\
  {									\
    ENAME##_cell_t _next PUK_LIST_ALIGN; /**< @internal next cell */	\
    ENAME##_cell_t _prev PUK_LIST_ALIGN; /**< @internal previous cell */ \
  } ENAME##_list_link;
  
/**@hideinitializer
 * The effective type declaration for lists */
#define PUK_LIST_CREATE_TYPE(ENAME, DECL)				\
  /** type for list cells */						\
  struct ENAME##_s							\
  {									\
    PUK_LIST_LINK(ENAME);						\
    DECL;								\
  };									\
  /** automatic Puk type: defines ENAME##_t */				\
  typedef struct ENAME##_s* ENAME##_t;

/** set cell link to NULL */
#define PUK_LIST_CELL_CLEAR(ENAME, ELT)					\
  { (ELT)->ENAME##_list_link._next = NULL;				\
    (ELT)->ENAME##_list_link._prev = NULL; }
/** check cell link to be NULL */
#define PUK_LIST_CHECK_NULL(ENAME, ELT)					\
  {									\
    assert((ELT)->ENAME##_list_link._next == NULL);			\
    assert((ELT)->ENAME##_list_link._prev == NULL);			\
  }

#ifdef PUK_DEBUG
#define PUK_LIST_CELL_INIT(ENAME, ELT)  PUK_LIST_CELL_CLEAR(ENAME, ELT)
#define PUK_LIST_CELL_CHECK(ENAME, ELT) PUK_LIST_CHECK_NULL(ENAME, ELT)
#define PUK_LIST_RESET(LIST)			\
  {						\
    (LIST)->_head = NULL;			\
    (LIST)->_tail = NULL;			\
  }
#else
#define PUK_LIST_CELL_INIT(ENAME, ELT)
#define PUK_LIST_CELL_CHECK(ENAME, ELT)
#define PUK_LIST_RESET(LIST)
#endif /* PUK_DEBUG */

#define PUK_LIST_CREATE_FUNCS(ENAME)					\
  __attribute__((__unused__))						\
  static inline void ENAME##_list_cell_init(ENAME##_cell_t e)		\
  {									\
    PUK_LIST_CELL_INIT(ENAME, e);					\
  }									\
  static inline void ENAME##_list_cell_setnull(ENAME##_cell_t e)	\
  {									\
    PUK_LIST_CELL_CLEAR(ENAME, e);					\
  }									\
  static inline void ENAME##_list_check_null(ENAME##_cell_t e)		\
  {									\
    PUK_LIST_CELL_CHECK(ENAME, e);					\
  }									\
  static inline int ENAME##_list_cell_isnull(ENAME##_cell_t e)		\
  {									\
    return((e->ENAME##_list_link._prev == NULL) &&			\
	   (e->ENAME##_list_link._next ==NULL));			\
  }									\
  /** @internal */							\
  __attribute__((__unused__))						\
  static inline ENAME##_cell_t ENAME##_new(void)			\
  { ENAME##_cell_t e = padico_malloc(sizeof(ENAME##_cell_content_t));	\
    ENAME##_list_cell_init(e);						\
    return e; }								\
  /** @internal */							\
  __attribute__((__unused__))						\
  static inline void ENAME##_delete(ENAME##_cell_t e)			\
  { padico_free(e); }							\
  /** @internal */							\
  __attribute__((__unused__))						\
  static inline void ENAME##_list_push_front(ENAME##_list_t l, ENAME##_cell_t e) \
  { ENAME##_list_check_null(e);						\
    if(l->_tail == NULL) l->_tail = e;					\
    else l->_head->ENAME##_list_link._prev = e;				\
    e->ENAME##_list_link._prev = NULL;					\
    e->ENAME##_list_link._next = l->_head; l->_head = e; }		\
  /** @internal */							\
  __attribute__((__unused__))						\
  static inline void ENAME##_list_push_back(ENAME##_list_t l, ENAME##_cell_t e) \
  {  ENAME##_list_check_null(e);					\
    if(l->_head == NULL) l->_head = e; else l->_tail->ENAME##_list_link._next = e; \
    e->ENAME##_list_link._next = NULL;					\
    e->ENAME##_list_link._prev = l->_tail; l->_tail = e; }		\
  /** @internal insert e after r */					\
  __attribute__((__unused__))						\
  static inline void ENAME##_list_insert_after(ENAME##_list_t l, ENAME##_cell_t r, ENAME##_cell_t e) \
  { ENAME##_list_check_null(e);						\
    e->ENAME##_list_link._next = r->ENAME##_list_link._next;		\
    e->ENAME##_list_link._prev = r;					\
    if(r->ENAME##_list_link._next == NULL) l->_tail = e;		\
    else r->ENAME##_list_link._next->ENAME##_list_link._prev = e;	\
    r->ENAME##_list_link._next = e; }					\
  /** @internal */							\
  __attribute__((__unused__))						\
  static inline ENAME##_cell_t ENAME##_list_front(ENAME##_list_t l)	\
  { return l->_head; }							\
  /** @internal */							\
  __attribute__((__unused__))						\
  static inline ENAME##_cell_t ENAME##_list_back(ENAME##_list_t l)	\
  { return l->_tail; }							\
  __attribute__((__unused__))						\
  static inline struct ENAME##_list_s ENAME##_list_null(void)		\
  { return (struct ENAME##_list_s){ ._head = NULL, ._tail = NULL }; }	\
  /** @internal */							\
  __attribute__((__unused__))						\
  static inline void ENAME##_list_init(ENAME##_list_t l)		\
  { *l = ENAME##_list_null(); }						\
  /** internal */							\
  static inline void ENAME##_list_destroy(ENAME##_list_t l)		\
  { PUK_LIST_RESET(l); }						\
  /** @internal */							\
  __attribute__((__unused__))						\
  static inline ENAME##_list_t ENAME##_list_new(void)			\
  { ENAME##_list_t l = padico_malloc(sizeof(struct ENAME##_list_s));	\
    ENAME##_list_init(l);						\
    return l; }								\
  /** @internal */							\
  __attribute__((__unused__))						\
  static inline int ENAME##_list_empty(ENAME##_list_t l)		\
  { return (l->_head == NULL); }					\
  /** @internal */							\
  __attribute__((__unused__))						\
  static inline void ENAME##_list_delete(ENAME##_list_t l)		\
  { padico_free(l); }							\
  /** @internal */							\
  __attribute__((__unused__))						\
  static inline int ENAME##_list_contains(ENAME##_list_t l, ENAME##_cell_t e) \
  { ENAME##_itor_t i;							\
    for(i = l->_head; i; i = i->ENAME##_list_link._next)		\
      {									\
	if(i == e) { return 1; }					\
      }									\
    return 0;								\
  }									\
  /** @internal */							\
  __attribute__((__unused__))						\
  static inline void ENAME##_list_remove(ENAME##_list_t l, ENAME##_cell_t e) \
  { assert(ENAME##_list_contains(l, e));				\
    const ENAME##_cell_t p = e->ENAME##_list_link._prev;		\
    if(p) p->ENAME##_list_link._next = e->ENAME##_list_link._next;	\
    else l->_head = e->ENAME##_list_link._next;				\
    if(e->ENAME##_list_link._next) e->ENAME##_list_link._next->ENAME##_list_link._prev = p; \
    else l->_tail = p;							\
    ENAME##_list_cell_init(e); }					\
  /** @internal */							\
  __attribute__((__unused__))						\
  static inline ENAME##_cell_t ENAME##_list_pop_front(ENAME##_list_t l)	\
  { ENAME##_cell_t e = ENAME##_list_front(l);				\
    if(e) ENAME##_list_remove(l, e);					\
    return e; }								\
  /** @internal */							\
  __attribute__((__unused__))						\
  static inline ENAME##_itor_t ENAME##_list_begin(ENAME##_list_t l)	\
  { return l->_head; }							\
  /** @internal */							\
  __attribute__((__unused__))						\
  static inline ENAME##_itor_t ENAME##_list_rbegin(ENAME##_list_t l)	\
  { return l->_tail; }							\
  /** @internal */							\
  __attribute__((__unused__))						\
  static inline ENAME##_itor_t ENAME##_list_end(ENAME##_list_t l)	\
  { return NULL; }							\
  /** @internal */							\
  __attribute__((__unused__))						\
  static inline ENAME##_itor_t ENAME##_list_rend(ENAME##_list_t l)	\
  { return NULL; }							\
  /** @internal */							\
  __attribute__((__unused__))						\
  static inline ENAME##_itor_t ENAME##_list_next(ENAME##_itor_t i)	\
  { return i->ENAME##_list_link._next; }				\
  /** @internal */							\
  __attribute__((__unused__))						\
  static inline ENAME##_itor_t ENAME##_list_rnext(ENAME##_itor_t i)	\
  { return i->ENAME##_list_link._prev; }				\
  /** @internal */							\
  __attribute__((__unused__))						\
  static inline int ENAME##_list_size(ENAME##_list_t l)			\
  { ENAME##_itor_t i=l->_head;						\
    int k = 0;								\
    while(i!=NULL){k++;i=i->ENAME##_list_link._next;}			\
    return k; }								

#define puk_list_foreach(ENAME, ITOR, LIST)				\
  for((ITOR)=(LIST)->_head; (ITOR); (ITOR)=(ITOR)->ENAME##_list_link._next)

#define puk_list_foreach_safe(ENAME, ITOR, ITOR2, LIST)			\
  for((ITOR)=(LIST)->_head, (ITOR2)=((ITOR)?(ITOR)->ENAME##_list_link._next:NULL); (ITOR); (ITOR)=(ITOR2), (ITOR2)=((ITOR)?(ITOR)->ENAME##_list_link._next:NULL))

#define PUK_LIST_FIND(ENAME, ITOR, LIST, COND)			\
  puk_list_foreach(ENAME, ITOR, LIST) { if((COND)) break; }

/** @remarks Puk_list how-to
 * PUK_LIST_DECLARE_TYPE(FOO) declares a FOO_t type and a FOO_list_t type
 * FOO_t is an element, FOO_list_t is a list of FOO_t elements
 * PUK_LIST_CREATE_TYPE(FOO, declrations) creates FOO_t and FOO-list_t types
 * and the related functions.
 * Example:
 * PUK_LIST_CREATE_TYPE(FOO, int a; float b;)
 * functions:
 *   FOO_t      FOO_new(void);
 *   void       FOO_delete(FOO_t);
 *   FOO_list_t FOO_list_new(void);
 *   void       FOO_list_delete(FOO_list_t);
 *   int        FOO_list_empty(FOO_list_t);
 *   void       FOO_list_remove(FOO_list_t, FOO_t);
 *   void       FOO_list_push_back(FOO_list_t, FOO_t);
 *   void       FOO_list_push_front(FOO_list_t, FOO_t);
 *   FOO_t      FOO_list_back(FOO_list_t);
 *   FOO_t      FOO_list_front(FOO_list_t);
 */


#endif /* PUK_LIST_H */

