/** @file
 * @brief Puk priority list, implemented as pairing heap
 * @ingroup Puk
 */

#ifndef PUK_PRIO_H
#define PUK_PRIO_H

/** implement priority queue as a pairing heap */
#define PUK_PRIO_PAIRING_HEAP
#undef PUK_PRIO_LIST

#define PUK_PRIO_TYPE(ENAME, DECL)		\
  PUK_PRIO_DECLARE_TYPE(ENAME)			\
  PUK_PRIO_CREATE_TYPE(ENAME, DECL)		\
  PUK_PRIO_CREATE_FUNCS(ENAME)

#define PUK_PRIO_CREATE_TYPE(ENAME, DECL)	\
  struct ENAME##_prio_cell_s			\
  {						\
    PUK_PRIO_LINK(ENAME);			\
    DECL;					\
  };

#define PUK_PRIO_DECLARE_TYPE(ENAME)				\
  PUK_PRIO_DECLARE_TYPE2(ENAME, struct ENAME##_prio_cell_s)

#if defined(PUK_PRIO_LIST)

/* ** priority queue as flat list ************************** */

#define PUK_PRIO_DECLARE_TYPE2(ENAME, CELL)			\
  PUK_LIST_DECLARE_TYPE2(ENAME##_prio_cell, CELL);		\
  typedef CELL*ENAME##_prio_cell_t;				\
  typedef struct ENAME##_prio_list_s*ENAME##_prio_list_t;	\
  struct ENAME##_prio_list_s					\
  {								\
    struct ENAME##_prio_cell_list_s _cells;			\
  };								\

#define PUK_PRIO_LINK(ENAME)			\
  int ENAME##_prio;				\
  PUK_LIST_LINK(ENAME##_prio_cell)

#define PUK_PRIO_CREATE_FUNCS(ENAME)					\
  PUK_LIST_CREATE_FUNCS(ENAME##_prio_cell)				\
  static inline void ENAME##_prio_list_init(struct ENAME##_prio_list_s*l) \
  {									\
    ENAME##_prio_cell_list_init(&l->_cells);				\
  }									\
  static inline void ENAME##_prio_list_insert(ENAME##_prio_list_t l, int prio, ENAME##_prio_cell_t c) \
  {									\
    c->ENAME##_prio = prio;						\
    ENAME##_prio_cell_itor_t i = ENAME##_prio_cell_list_rbegin(&l->_cells); \
    while((i != NULL) && (prio > i->ENAME##_prio))			\
      {									\
	i = ENAME##_prio_cell_list_rnext(i);				\
      }									\
    if(i)								\
      ENAME##_prio_cell_list_insert_after(&l->_cells, i, c);		\
    else								\
      ENAME##_prio_cell_list_push_front(&l->_cells, c);			\
  }									\
  static inline ENAME##_prio_cell_t ENAME##_prio_list_pop(ENAME##_prio_list_t l) \
  {									\
    return ENAME##_prio_cell_list_pop_front(&l->_cells);			\
  }									\
  static inline ENAME##_prio_cell_t ENAME##_prio_list_top(ENAME##_prio_list_t l) \
  {									\
    return ENAME##_prio_cell_list_begin(&l->_cells);			\
  }									\
  static inline int ENAME##_prio_cell_get_prio(ENAME##_prio_cell_t c)	\
  {									\
    return c->ENAME##_prio;						\
  }
       
#elif defined(PUK_PRIO_PAIRING_HEAP)

/* ** priority queue as pairing heap *********************** */

/** types for pairing heap */
#define PUK_PRIO_DECLARE_TYPE2(ENAME, CELL)			\
  typedef CELL*ENAME##_prio_cell_t;				\
  typedef struct ENAME##_prio_list_s*ENAME##_prio_list_t;	\
  struct ENAME##_prio_list_s					\
  {								\
    ENAME##_prio_cell_t _root;					\
  };								\

/** link for pairing heap */
#define PUK_PRIO_LINK(ENAME)						\
  struct ENAME##_prio_link_s						\
  {									\
    int _prio;								\
    ENAME##_prio_cell_t _child						\
      __attribute__((aligned((sizeof(void*)))));			\
    ENAME##_prio_cell_t _sibling					\
      __attribute__((aligned((sizeof(void*)))));			\
  } ENAME##_prio_link;


/** func for pairing heap */
#define PUK_PRIO_CREATE_FUNCS(ENAME)					\
									\
  /** merge 2 heaps */							\
  static inline ENAME##_prio_cell_t ENAME##_prio_list_meld(ENAME##_prio_cell_t a, ENAME##_prio_cell_t b) \
  {									\
    if(a == NULL)							\
      {									\
	return b;							\
      }									\
    else if( b == NULL)							\
      {									\
	return a;							\
      }									\
    else if(a->ENAME##_prio_link._prio < b->ENAME##_prio_link._prio)	\
      {									\
	a->ENAME##_prio_link._sibling = b->ENAME##_prio_link._child;	\
	b->ENAME##_prio_link._child = a;				\
	return b;							\
      }									\
    else								\
      {									\
	b->ENAME##_prio_link._sibling = a->ENAME##_prio_link._child;	\
	a->ENAME##_prio_link._child = b;				\
	return a;							\
      }									\
  }									\
  /** @internal */							\
  static inline ENAME##_prio_cell_t ENAME##_prio_list_merge_binary(ENAME##_prio_cell_t*c, int depth) \
  {									\
    if(*c == NULL)							\
      {									\
	return NULL;							\
      }									\
    else if(depth == 0)							\
      {									\
	ENAME##_prio_cell_t a = *c;					\
	*c = (*c)->ENAME##_prio_link._sibling;				\
	a->ENAME##_prio_link._sibling = NULL;				\
	return a;							\
      }									\
    else								\
      {									\
	ENAME##_prio_cell_t a = ENAME##_prio_list_merge_binary(c, depth - 1); \
	ENAME##_prio_cell_t b = ENAME##_prio_list_merge_binary(c, depth - 1); \
	return ENAME##_prio_list_meld(a, b);				\
      }									\
  }									\
									\
  /** @internal */ 							\
  static inline ENAME##_prio_cell_t ENAME##_prio_list_merge_exp(ENAME##_prio_cell_t*c) \
  {									\
    ENAME##_prio_cell_t a = ENAME##_prio_list_merge_binary(c, 0);	\
    int depth = 0;							\
    while(*c)								\
      {									\
	a = ENAME##_prio_list_meld(a, ENAME##_prio_list_merge_binary(c, depth)); \
	depth++;							\
      }									\
    return a;								\
  }									\
  /** return top cell */						\
  static inline ENAME##_prio_cell_t ENAME##_prio_list_top(ENAME##_prio_list_t l) \
  {									\
    return l->_root;							\
  }									\
  /** remove top cell */						\
  static inline ENAME##_prio_cell_t ENAME##_prio_list_pop(ENAME##_prio_list_t l) \
  {									\
    if(l->_root == NULL)						\
      {									\
	return NULL;							\
      }									\
    else								\
      {									\
	ENAME##_prio_cell_t top = l->_root;				\
	l->_root = ENAME##_prio_list_merge_exp(&l->_root->ENAME##_prio_link._child); \
	return top;							\
      }									\
  }									\
  /** insert a cell in the prio list */					\
  static inline void ENAME##_prio_list_insert(ENAME##_prio_list_t l, int prio, ENAME##_prio_cell_t c) \
  {									\
    c->ENAME##_prio_link._prio = prio;					\
    c->ENAME##_prio_link._child = NULL;					\
    c->ENAME##_prio_link._sibling = NULL;				\
    l->_root = ENAME##_prio_list_meld(l->_root, c);			\
  }									\
  /** allocate new prio list */						\
  static inline void ENAME##_prio_list_init(struct ENAME##_prio_list_s*l) \
  {									\
    l->_root = NULL;							\
  }									\
  static inline int ENAME##_prio_cell_get_prio(ENAME##_prio_cell_t c)	\
  {									\
    return c->ENAME##_prio_link._prio;					\
  }

#endif /* PUK_PRIO_PIRING_HEAP */

#endif /* PUK_PRIO_H */
