dnl -*- mode: Autoconf;-*-

AC_DEFUN([AC_PUK_OUT],
	 [ dnl --- Puk/
	   out_mk="${out_mk} Makefile Puk.pc Puk.gdbinit"
	   dnl --- Puk/ModulesTools/
	   out_mk="${out_mk} ModulesTools/Makefile ModulesTools/padico-core-wrapper.sh"
	   AC_PADICO_OUT_SH([ModulesTools/padico-mkmod])
	   AC_PADICO_OUT_SH([ModulesTools/padico-cc])
	   AC_PADICO_OUT_SH([ModulesTools/padico-cdep])
	 ])

dnl -- Puk root path
dnl --
AC_DEFUN([AC_PUK_ROOT],
	 [
	   AC_REQUIRE([AC_PADICO_PACKAGE_ROOT])
	   AC_MSG_CHECKING([for PUK_ROOT])
	   puk_root=${prefix}
           AC_DEFINE_UNQUOTED(PUK_ROOT, "${puk_root}/", [Puk root install directory])
	   AC_MSG_RESULT([${puk_root}])
	 ])

# ## check how to disable std lib linkage
# used by padico-mkmod
AC_DEFUN([AC_PUK_NOSTDLIB],
	 [ AC_REQUIRE([AC_PADICO_LINKER])
	   AC_MSG_CHECKING([for nostdlib ld flag])
	   case ${LD} in
	     *c++*|*g++*|*gcc*|*clang*|*icc*) LDNOSTDLIB="";; # LDNOSTDLIB="-nostdlib";;
	     'CC')  LDNOSTDLIB="-z muldefs";;
	     *) AC_MSG_ERROR([Unknown C++ linker: ${LD}. Stop.])
	   esac
	   AC_MSG_RESULT([${LDNOSTDLIB}])
	   AC_SUBST(LDNOSTDLIB)
	 ])



# ## how to convert a static library into a shared object file
# used by padico-mkmod
AC_DEFUN([AC_PUK_ATOSO],
	 [ AC_REQUIRE([AC_PADICO_COMPILER])
	   AC_MSG_CHECKING([whether static libraries can be converted into dynamic libraries ])
	   libm_a=`${CC} -print-file-name=libm.a`
	   if test -r ${libm_a} ; then
	     _LOCALLIBA=${libm_a}
	   else
	     _LOCALLIBA=`echo /usr/lib/lib*.a | cut -f 1 -d ' '`
	     if test ! -r ${_LOCALLIBA} ; then
	       AC_MSG_WARN([ Unable to find a lib.a in /usr/lib ])
	     fi
	   fi
	   _LOCALLIBSO=test.so
	   atoso_flags_gnu="-shared -export-dynamic -whole-archive -fPIC"
	   atoso_flags_bsd="-G z allextract"
	   ATOSO_OK="yes"
	   if ( ld ${atoso_flags_gnu} ${_LOCALLIBA} -o ${_LOCALLIBSO} 1> /dev/null 2>&1 ); then
	     ATOSO="ld ${atoso_flags_gnu}";
	   elif ( gnu-ld ${atoso_flags_gnu} ${_LOCALLIBA} -o ${_LOCALLIBSO} 1> /dev/null 2>&1 ); then
	     ATOSO="gnu-ld ${atoso_flags_gnu}";
	   elif ( ld ${atoso_flags_bsd} ${_LOCALLIBA} -o ${_LOCALLIBSO}  1> /dev/null 2>&1 ); then
	     ATOSO="ld ${atoso_flags_bsd}";
	   else
	     ATOSO="";
	     ATOSO_OK="no";
	   fi
	   if test -r ${_LOCALLIBSO} ; then rm ${_LOCALLIBSO}; fi
	   AC_MSG_RESULT([${ATOSO_OK}: ${ATOSO} ])
	   AC_SUBST(ATOSO)
	   AC_SUBST(ATOSO_OK)
	 ])

# ## required and optional features
AC_DEFUN([AC_PUK_FEATURES],
	 [ AC_REQUIRE([AC_PADICO_COMPILER])
	   AC_REQUIRE([AC_PADICO_DEBUG])
	   if test "x${ENABLE_DEBUG_FLAG}" = "xyes"; then
	      AC_DEFINE([PUK_DEBUG], [1], [whether Puk is built in DEBUG])
	   fi

	   dnl - required headers
	   AC_CHECK_HEADERS(dlfcn.h setjmp.h errno.h sys/stat.h sys/wait.h,
	     [],
	     AC_MSG_ERROR([Missing required header file]))

	   AC_CHECK_HEADER(elf.h,
	     [ HAVE_ELF_H="yes"
	       AC_DEFINE(HAVE_ELF_H, 1, [Define if you have the <elf.h> header file.])],
	     [ HAVE_ELF_H="no"])

	   AC_CHECK_HEADER(stdint.h,
	     [ HAVE_STDINT_H="yes"
	       AC_DEFINE(HAVE_STDINT_H, 1, [Define if you have the <stdint.h> header file.])
	       AC_DEFINE(PUK_HAVE_STDINT_H, 1, [Define if you have the <stdint.h> header file.])
	     ],
	     [ HAVE_STDINT_H="no"])

	   AC_CHECK_HEADER(valgrind/valgrind.h,
	     [ HAVE_VALGRIND_H="yes"
	       AC_DEFINE(HAVE_VALGRIND_H, 1, [Define if you have the <valgrind/valgrind.h> header file.])
	       AC_DEFINE(PUK_HAVE_VALGRIND_H, 1, [Define if you have the <valgrind/valgrind.h> header file.])
	     ],
	     [ HAVE_VALGRIND_H="no" ])

	   AC_CHECK_FUNCS([ setenv ])

	   AC_SEARCH_LIBS(clock_gettime, [c rt],
	     [ AC_DEFINE(HAVE_CLOCK_GETTIME, 1, [have clock_gettime])
	       AC_DEFINE(PUK_HAVE_CLOCK_GETTIME, 1, [have clock_gettime]) ], [])
	   AC_CHECK_DECLS(mach_absolute_time,
	     [ AC_DEFINE(HAVE_MACH_ABSOLUTE_TIME, 1, [have mach_absolute_time declaration])
	       AC_DEFINE(PUK_HAVE_MACH_ABSOLUTE_TIME, 1, [have mach_absolute_time]) ], [], 
		[[
			#include <mach/mach_time.h>
		]])
	 ])
