dnl -*- mode: Autoconf;-*-

dnl -- Optionnal features
dnl --------------------------------------------------------

AC_DEFUN([AC_PADICO_CHECK_ZLIB],
	 [
	   AC_CHECK_HEADER(zlib.h, 
 	        [ HAVE_ZLIB_H="yes"
	          HAVE_ZLIB="yes"
	          AC_DEFINE(HAVE_ZLIB_H, 1, [Define if you have the <zlib.h> header file.])],
	        [ HAVE_ZLIB_H="no"
	          HAVE_ZLIB="no" ])
	   AC_SUBST(HAVE_ZLIB)
	 ])

AC_DEFUN([AC_PADICO_CHECK_BZLIB],
	 [
	   AC_CHECK_HEADER(bzlib.h, 
	        [ HAVE_BZLIB_H="yes"
	          HAVE_BZLIB="yes"
	          AC_DEFINE(HAVE_BZLIB_H, 1, [Define if you have the <bzlib.h> header file.])],
	        [ HAVE_BZLIB_H="no"
	          HAVE_BZLIB="no" ])
	   AC_SUBST(HAVE_BZLIB)
 	 ])

AC_DEFUN([AC_PADICO_CHECK_LZ4],
	 [
	   AC_CHECK_HEADER(lz4.h, 
	        [ HAVE_LZ4_H="yes"
	          HAVE_LIBLZ4="yes"
	          AC_DEFINE(HAVE_LZ4_H, 1, [Define if you have the <lz4.h> header file.])],
	        [ HAVE_LZ4_H="no"
	          HAVE_LIBLZ4="no" ])
  	   AC_SUBST(HAVE_LIBLZ4)
	 ])

AC_DEFUN([AC_PADICO_CHECK_LIBCURL],
	 [
	   AC_REQUIRE([PKG_PROG_PKG_CONFIG])
           PKG_CHECK_MODULES([libcurl], [libcurl], [HAVE_LIBCURL=yes], [HAVE_LIBCURL=no])
	   if test x${HAVE_LIBCURL} = xyes; then
	     AC_CHECK_HEADER(curl/curl.h, [ ], [ AC_MSG_ERROR([unusable libcurl]) ])
	   fi
	   AC_SUBST([HAVE_LIBCURL])
	 ])

AC_DEFUN([AC_PADICO_CHECK_JANSSON],
	 [
	   AC_REQUIRE([PKG_PROG_PKG_CONFIG])
           PKG_CHECK_MODULES([jansson], [jansson], [HAVE_JANSSON=yes], [HAVE_JANSSON=no])
	   if test x${HAVE_JANSSON} = xyes; then
	     AC_CHECK_HEADER(jansson.h, [ ], [ AC_MSG_ERROR([unusable jansson]) ])
	   fi
	   AC_SUBST([HAVE_JANSSON])
	 ])

# ## checks for hwloc
# input: need_hwloc=yes|""
# output: with_hwloc, requires_hwloc, hwloc_CFLAGS|LIBS
AC_DEFUN([AC_PADICO_CHECK_HWLOC],
	 [
	   AC_REQUIRE([PKG_PROG_PKG_CONFIG])
	   AC_ARG_WITH([hwloc],
	               [AS_HELP_STRING([--with-hwloc], [use hwloc @<:@default=auto@:>@] )])
           if test "x${need_hwloc}" = "xyes"; then
  	     with_hwloc=yes
	   fi
	   HAVE_HWLOC=no
	   if test "x${with_hwloc}" != "xno"; then
 	     PKG_CHECK_MODULES([hwloc], [ hwloc ], [HAVE_HWLOC=yes], [HAVE_HWLOC=no])
	   fi
	   if test "x${with_hwloc}" = "xyes" -a "${HAVE_HWLOC}" = "no"; then
	     AC_MSG_ERROR([ required library hwloc not found.])
 	   fi
	   if test "${HAVE_HWLOC}" = "yes"; then
	     with_hwloc=yes
	     requires_hwloc=hwloc
	     AC_DEFINE([HAVE_HWLOC], [1], [whether hwloc is present])
	   fi
	   AC_SUBST([requires_hwloc])
	   AC_SUBST([with_hwloc])
	 ])


dnl -- check for Infiniband (ibverbs)
dnl --
AC_DEFUN([AC_PADICO_CHECK_INFINIBAND],
  [
    HAVE_INFINIBAND=no
    with_ibverbs=check
    AC_ARG_WITH([ibverbs],
      [AS_HELP_STRING([--with-ibverbs], [use Infiniband ibverbs @<:@default=check@:>@] )])
    if test "x${with_ibverbs}" != "xno"; then
      save_CPPFLAGS="${CPPFLAGS}"
      save_LIBS="${LIBS}"
      if test "x${with_ibverbs}" != "xcheck" -a "${with_ibverbs}" != "xyes"; then
        IBHOME="${with_ibverbs}"
      fi
      if test "x$IBHOME" != "x" -a "x${IBHOME}" != "x/usr"; then
    	ibverbs_CPPLAGS="-I${IBHOME}/include"
        ibverbs_LIBS="-Wl,-rpath,${IBHOME}/lib -L${IBHOME}/lib"
      fi
      CPPFLAGS="${CPPFLAGS} ${ibverbs_CPPFLAGS}"
      LIBS="${LIBS} ${ibverbs_LIBS}"
      AC_CHECK_HEADER(infiniband/verbs.h, [ HAVE_INFINIBAND_H=yes ], [ HAVE_INFINIBAND_H=no  ])
      AC_CHECK_LIB(ibverbs, ibv_open_device, [ HAVE_LIBIBVERBS=yes ], [ HAVE_LIBIBVERBS=no ])
      if test ${HAVE_INFINIBAND_H} = yes -a ${HAVE_LIBIBVERBS} = yes; then
        HAVE_INFINIBAND=yes
	ibverbs_LIBS="${ibverbs_LIBS} -libverbs"
      else
        HAVE_INFINIBAND=no
        ibverbs_CPPFLAGS=
    	ibverbs_LIBS=
        if test "x${with_ibverbs}" != "xcheck"; then
          AC_MSG_FAILURE([--with-ibverbs was given, but infiniband/verbs.h is not found])
        fi
      fi
      CPPFLAGS="${save_CPPFLAGS}"
      LIBS="${save_LIBS}"
    fi    
    AC_SUBST(HAVE_INFINIBAND)
    AC_SUBST(ibverbs_CPPFLAGS)
    AC_SUBST(ibverbs_LIBS)
  ])

dnl -- check for Intel PSM
dnl --
AC_DEFUN([AC_PADICO_CHECK_PSM],
  [
    HAVE_PSM=no
    AC_ARG_WITH([psm],
	        [AS_HELP_STRING([--with-psm],
		 [use Intel Performance Scaled Messaging (PSM) @<:@default=check@:>@] )],
	         [], [ with_psm=check ])
    if test "x${with_psm}" != "xno"; then
      save_CPPFLAGS="${CPPFLAGS}"
      save_LIBS="${LIBS}"
      if test "x${with_psm}" != "xyes" -a "x${with_psm}" != "xcheck"; then
        PSM_DIR="${with_psm}"
      fi
      if test "x${PSM_DIR}" != "x"; then
  	psm_CPPFLAGS="-I${PSM_DIR}/include"
	psm_LIBS="-Wl,-rpath,${PSM_DIR}/lib -L${PSM_DIR}/lib"
      fi
      CPPFLAGS="${CPPFLAGS} ${psm_CPPFLAGS}"
      LIBS="${LIBS} ${psm_LIBS}"
      AC_CHECK_HEADER(psm.h, [ HAVE_PSM_H=yes ], [ HAVE_PSM_H=no ])
      AC_SEARCH_LIBS(psm_init, [ psm psm_infinipath ], [ HAVE_LIBPSM=yes ], [ HAVE_LIBPSM=no ])
      if test ${HAVE_PSM_H} = yes -a ${HAVE_LIBPSM} = yes; then
        HAVE_PSM=yes
	psm_LIBS="${psm_LIBS} ${ac_cv_search_psm_init}"
      else
	HAVE_PSM=no
	psm_CPPFLAGS=
	psm_LIBS=
        if test "x${with_psm}" != "xcheck"; then
      	  AC_MSG_FAILURE([--with-psm was given but psm.h is not found.])
        fi
      fi
      CPPFLAGS="${save_CPPFLAGS}"
      LIBS="${save_LIBS}"
    fi
    AC_SUBST(HAVE_PSM)
    AC_SUBST([psm_CPPFLAGS])
    AC_SUBST([psm_LIBS])
  ])

dnl -- check for Intel PSM2
dnl --
AC_DEFUN([AC_PADICO_CHECK_PSM2],
  [
    HAVE_PSM2=no
    AC_ARG_WITH([psm2],
		[AS_HELP_STRING([--with-psm2],
	    	 [use Intel Performance Scaled Messaging 2 (PSM2) @<:@default=check@:>@] )],
		 [], [ with_psm2=check ])
    if test "x${with_psm2}" != "xno"; then
      save_CPPFLAGS="${CPPFLAGS}"
      save_LIBS="${LIBS}"
      if test "x${with_psm2}" != "xyes" -a "x${with_psm2}" != "xcheck"; then
        PSM2_DIR="${with_psm2}"
      fi
      if test "x${PSM2_DIR}" != "x"; then
      	psm2_CPPFLAGS="-I${PSM2_DIR}/include"
        psm2_LIBS="-Wl,-rpath,${PSM2_DIR}/lib -L${PSM2_DIR}/lib"
      fi
      CPPFLAGS="${CPPFLAGS} ${psm2_CPPFLAGS}"
      LIBS="${LIBS} ${psm2_LIBS}"
      AC_CHECK_HEADER(psm2.h, [ HAVE_PSM2_H=yes ], [ HAVE_PSM2_H=no ])
      AC_CHECK_LIB(psm2, psm2_init, [ HAVE_LIBPSM2=yes ], [ HAVE_LIBPSM2=no ])
      if test ${HAVE_PSM2_H} = yes -a ${HAVE_LIBPSM2} = yes; then
        HAVE_PSM2=yes
	psm2_LIBS="${psm2_LIBS} -lpsm2"
      else
	HAVE_PSM2=no
	psm2_CPPFLAGS=
	psm2_LIBS=
        if test "x${with_psm2}" != "xcheck"; then
      	  AC_MSG_FAILURE([--with-psm2 was given but psm2.h is not found.])
        fi
      fi
      CPPFLAGS="${save_CPPFLAGS}"
      LIBS="${save_LIBS}"
    fi
    AC_SUBST(HAVE_PSM2)
    AC_SUBST([psm2_CPPFLAGS])
    AC_SUBST([psm2_LIBS])
  ])

dnl -- check for OFI (libfabric)
dnl --
AC_DEFUN([AC_PADICO_CHECK_OFI],
  [
    HAVE_OFI=no
    AC_ARG_WITH([ofi],
		[AS_HELP_STRING([--with-ofi],
	    	 [use OpenFabric Interface (libfabric) @<:@default=check@:>@] )],
		 [], [ with_ofi=check ])
    if test "x${with_ofi}" != "xno"; then
      PKG_CHECK_MODULES([ofi], [libfabric], [HAVE_OFI=yes], [HAVE_OFI=no])
    fi
    if test "x${with_ofi}" = "xyes" -a "${HAVE_OFI}" = "no"; then
      AC_MSG_ERROR([ required library OFI (libfabric) not found.])
    fi
    if test "${HAVE_OFI}" = "yes"; then
      AC_CHECK_HEADER(rdma/fabric.h, [ HAVE_FABRIC_H=yes ], [ HAVE_FABRIC_H=no ])
      requires_ofi=libfabric
    fi
    AC_SUBST([HAVE_OFI])
    AC_SUBST([ofi_CFLAGS])
    AC_SUBST([ofi_LIBS])
  ])

dnl --- Types
dnl --------------------------------------------------------

# ## socklen_t
AC_DEFUN(AC_TYPE_SOCKLEN_T,
[AC_CACHE_CHECK([for socklen_t], ac_cv_type_socklen_t,
[
  AC_TRY_COMPILE(
  [#include <sys/socket.h>],
  [socklen_t len = 42; return len;],
  ac_cv_type_socklen_t=yes,
  ac_cv_type_socklen_t=no)
])
  if test $ac_cv_type_socklen_t = yes; then
    AC_DEFINE(HAVE_SOCKLEN_T, [], [whether socklen_t is defined in sys/socket.h])
  fi
])

# ## off64_t
AC_DEFUN([AC_TYPE_OFF64_T],
	 [ AC_CACHE_CHECK([for off64_t], ac_cv_type_off64_t,
[
  AC_TRY_COMPILE(
  [#define _LARGEFILE64_SOURCE
   #include <sys/types.h>],
  [off64_t off = 42;],
  ac_cv_type_off64_t=yes,
  ac_cv_type_off64_t=no)
])
  if test $ac_cv_type_off64_t = yes; then
    AC_DEFINE(HAVE_OFF64_T, [], [whether off64_t is defined in sys/types.h using _LARGEFILE64_SOURCE])
    largefilecflags=-D_LARGEFILE64_SOURCE
  fi
  AC_SUBST(largefilecflags)
])

# ## check base required types
AC_DEFUN([AC_PADICO_TYPES],
	 [ AC_REQUIRE([AC_PADICO_COMPILER])
	   AC_C_BIGENDIAN
	   AC_CHECK_SIZEOF(int)
	   AC_CHECK_SIZEOF(long)
	   AC_CHECK_SIZEOF(long long)
	   AC_CHECK_SIZEOF(ssize_t)
	   AC_TYPE_SOCKLEN_T
	   AC_TYPE_OFF_T
	   AC_TYPE_OFF64_T
	   AC_TYPE_SIZE_T
	   AC_TYPE_PID_T
	 ])

dnl --- libs
dnl --------------------------------------------------------

AC_DEFUN([AC_PADICO_LIBS],
	 [ AC_REQUIRE([AC_PADICO_LINKER])
	   AC_SEARCH_LIBS(dlopen,  [dl dlcompat c c++], [], [ AC_MSG_ERROR([Cannot find dlopen() ]) ])
	   AC_SEARCH_LIBS(dlclose, [dl dlcompat c c++], [], [ AC_MSG_ERROR([Cannot find dlclose() ]) ])
	   AC_SEARCH_LIBS(dlsym,   [dl dlcompat c c++], [], [ AC_MSG_ERROR([Cannot find dlsym() ]) ])

	   AC_SEARCH_LIBS(sched_yield,   [c rt], [], [ AC_MSG_ERROR([Cannot find sched_yield() ]) ])

	   AC_SEARCH_LIBS(socket, nsl socket, [], [ AC_MSG_ERROR([Cannot find socket() ]) ])
	   AC_SEARCH_LIBS(listen, nsl socket, [], [ AC_MSG_ERROR([Cannot find listen() ]) ])
	   AC_SEARCH_LIBS(accept, nsl socket, [], [ AC_MSG_ERROR([Cannot find accept() ]) ])
	   AC_SEARCH_LIBS(gethostbyname, nsl socket, [], [ AC_MSG_ERROR([Cannot find gethostbyname() ]) ])
   
	 ])


dnl --- Arguments
dnl --------------------------------------------------------

AC_DEFUN([AC_PADICO_OPTIMIZE],
	 [ ENABLE_OPTIMIZE_FLAG="yes";
	   AC_ARG_ENABLE(optimize,
	        	 [  --enable-optimize       Build with optimization compiler flags @<:@default: yes@:>@ ],
	                 [ if test $enableval = no; then
	                     ENABLE_OPTIMIZE_FLAG="no";
	                   fi
	                 ],
	                 [])
	   AC_SUBST(ENABLE_OPTIMIZE_FLAG)
	 ])

AC_DEFUN([AC_PADICO_DEBUG],
	 [ ENABLE_DEBUG_FLAG="no";
	   AC_ARG_ENABLE(debug,
	   	         [  --enable-debug          Build with debug flags @<:@default: no@:>@],
	                 [ if test $enableval = yes; then 
	                     ENABLE_DEBUG_FLAG="yes";
	                   fi
	                 ],
	                 [])
	   AC_SUBST(ENABLE_DEBUG_FLAG)
 	 ])

AC_DEFUN([AC_PADICO_TRACE],
	 [ AC_ARG_ENABLE(trace,
	                 [ --enable-trace ], 
	                 [ if test "x$enableval" = "xyes"; then
	                     AC_DEFINE(PADICO_TRACE_ON, 1, [enable Padico trace system])
	                   fi],
	                 [])
	 ])

