/*
 * NewMadeleine
 * Copyright (C) 2006-2019 (see AUTHORS file)
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or (at
 * your option) any later version.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 */

#include <stdlib.h>
#include <stdio.h>
#include <stdint.h>
#include <string.h>
#include <unistd.h>
#include <nm_launcher_interface.h>

#include "../common/nm_examples_helper.h"
#include "sync_clock.h"


int main(int argc, char**argv)
{
  int rank, worldsize;
  static nm_session_t p_session = NULL;
  nm_gate_t gate;
  uint64_t t_ref_server, mid;

  nm_launcher_init(&argc, argv);
  MPI_Init(&argc,&argv);
  nm_launcher_session_open(&p_session, "nm_example");
  nm_launcher_get_rank(&rank);
  nm_launcher_get_size(&worldsize);

  Process_Sync_Offset = 0;
  sync_mpi();

  MPI_Barrier(MPI_COMM_WORLD);

  if(rank == 0) // server
  {
    for(int i = 1; i < worldsize; i++)
    {
      nm_launcher_get_gate(i, &gate);

      uint64_t t1, t2;

      t1 = Timer_tsc();
      nm_sr_send(p_session, gate, 1, &t1, sizeof(t1));
      nm_sr_recv(p_session, gate, 1, &mid, sizeof(mid));
      t2 = Timer_tsc();

      printf("[%d]   t1 = %lu us\n", i, t1);
      printf("[%d] tmid = %lu us\n", i, mid);
      printf("[%d]   t2 = %lu us\n", i, t2);
      
      if(t1 <= mid && mid <= t2)
      {
        printf("[%d] ok: %f\n", i, ((double) mid - (double) t1)/((double) t2 - (double) t1));
      }
      else
      {
        printf("[%d] bad\n", i);
      }
    }
  }
  else
  {
    uint64_t t_mid;
    nm_launcher_get_gate(0, &gate);

    nm_sr_recv(p_session, gate, 1, &t_ref_server, sizeof(t_ref_server));
    t_mid = Timer_tsc();
    nm_sr_send(p_session, gate, 1, &t_mid, sizeof(t_mid));
  }

  nm_launcher_exit();
  exit(0);
}
