/*
 * NewMadeleine
 * Copyright (C) 2014-2018 (see AUTHORS file)
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or (at
 * your option) any later version.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 */

#include <nm_public.h>
#include <nm_sendrecv_interface.h>
#include <nm_private.h>
#include <Padico/Puk.h>

#include "nm_coll_interface.h"
#include "nm_coll_trees.h"

/* ********************************************************* */

/** maximum tag usable by enduser for p2p */
#define NM_COLL_TAG_MASK_P2P      0x7FFFFFFF
/** base tag used for collectives */
#define NM_COLL_TAG_BASE          0xFF000000
/** comm_create phase 1 */
#define NM_COLL_TAG_COMM_CREATE_1 ( NM_COLL_TAG_BASE | 0x04 )
/** comm_create phase 2 */
#define NM_COLL_TAG_COMM_CREATE_2 ( NM_COLL_TAG_BASE | 0x05 )

/* ********************************************************* */

struct nm_coll_tree_status_s
{
  nm_session_t p_session;
  nm_group_t p_group;
  nm_tag_t tag;
  struct nm_coll_tree_info_s tree;
  int*children;
  nm_sr_request_t*children_requests;
  nm_sr_request_t parent_request;
  int pending_reqs;
  int s;
  nm_cond_status_t status;
};

static inline void nm_coll_tree_status_init(struct nm_coll_tree_status_s*p_status, nm_session_t p_session,
                                            nm_group_t p_group, int root, int self, nm_tag_t tag)
{
  const enum nm_coll_tree_kind_e kind = NM_COLL_TREE_BINOMIAL;
  p_status->p_session = p_session;
  p_status->p_group = p_group;
  p_status->tag = tag;
  nm_coll_tree_init(&p_status->tree, kind, nm_group_size(p_group), self, root);
  p_status->children_requests = malloc(p_status->tree.max_arity * sizeof(nm_sr_request_t));
  p_status->children = malloc(p_status->tree.max_arity * sizeof(int));
  nm_cond_init(&p_status->status, 0);
  p_status->pending_reqs = 0;
  p_status->s = 0;
}

static inline void nm_coll_tree_status_wait(struct nm_coll_tree_status_s*p_status)
{
  nm_cond_wait(&p_status->status, 1, p_status->p_session->p_core);
  free(p_status->children_requests);
  free(p_status->children);
}

static inline void nm_coll_tree_send(struct nm_coll_tree_status_s*p_coll_tree, int dest,
                                     struct nm_data_s*p_data, nm_sr_request_t*p_req,
                                     void(*p_notifier)(nm_sr_event_t, const nm_sr_event_info_t*, void*))
{
  assert(dest != -1);
  nm_gate_t p_gate = nm_group_get_gate(p_coll_tree->p_group, dest);
  nm_atomic_inc(&p_coll_tree->pending_reqs);
  nm_sr_isend_data(p_coll_tree->p_session, p_gate, p_coll_tree->tag, p_data, p_req);
  nm_sr_request_set_ref(p_req, p_coll_tree);
  nm_sr_request_monitor(p_coll_tree->p_session, p_req, NM_SR_EVENT_FINALIZED, p_notifier);
}

static inline void nm_coll_tree_recv(struct nm_coll_tree_status_s*p_coll_tree, int from,
                                     struct nm_data_s*p_data, nm_sr_request_t*p_req,
                                     void(*p_notifier)(nm_sr_event_t, const nm_sr_event_info_t*, void*))
{
  assert(from != -1);
  nm_gate_t p_gate = nm_group_get_gate(p_coll_tree->p_group, from);
  nm_atomic_inc(&p_coll_tree->pending_reqs);
  nm_sr_irecv_data(p_coll_tree->p_session, p_gate, p_coll_tree->tag, p_data, p_req);
  nm_sr_request_set_ref(p_req, p_coll_tree);
  nm_sr_request_monitor(p_coll_tree->p_session, p_req, NM_SR_EVENT_FINALIZED, p_notifier);
}
